VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Range"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Edanmo's RichEdit Control v3 - Range Class"
'*********************************************************************************************
'
' RichEdit Control
'
' Range class
'
'*********************************************************************************************
'
' Author: Eduardo A. Morcillo
' E-Mail: e_morcillo@yahoo.com
' Web Page: http://www.domaindlx.com/e_morcillo
'
' Distribution: You can freely use this code in your own applications but you
'               can't publish this code in a web site, online service, or any
'               other media, without my express permission.
'
' Usage: at your own risk.
'
' Tested on: Windows 98 + VB5
'
' History:
'           07/11/2000 - This code was released
'
'*********************************************************************************************
Option Explicit

Private m_Range As ITextRange     ' ITextRange object
Private m_Para As ParagraphFormat ' Paragraph format
Private m_Font As RichFont        ' Font

' ===== IDataObject_Implements Variables =====

' This object implements
' the IDataObject_Implements interface
Implements olelib2.IDataObject

' RTF text buffer used by
' the TextRTF property
Dim m_IDORTF As String

' Image object used by
' the InsertImage function
Dim m_Image As StdPicture

' ===== Enumerations =====

Enum reConstants
   reToggle = tomToggle          ' Toggles the format
   reUndefined = tomUndefined    ' Mixed formats
   reDefault = tomDefault        ' Default format
   reAutoColor = tomAutoColor    ' Default color
   reForward = tomForward        ' Moves forward
   reBackward = tomBackward      ' Moves backward
End Enum

' Valid units
Enum reUnits
   reChar = tomCharacter
   reWord = tomWord
   reSentence = tomSentence
   reParagraph = tomParagraph
   reLine = tomLine
   reScreen = tomScreen
   reSection = tomSection
   reColumn = tomColumn
   reRow = tomRow
   reWindow = tomWindow
   reCell = tomCell
   reCharFormat = tomCharFormat
   reParaFormat = tomParaFormat
   reTable = tomTable
   reEmbeddedObject = tomObject
End Enum

Enum reFindFlags
   reFindMatchWord = tomMatchWord
   reFindMatchCase = tomMatchCase
End Enum

Enum reChangeCaseFlags
   reCaseLower = tomLowerCase
   reCaseUpper = tomUpperCase
   reCaseTitle = tomTitleCase
   reCaseSentence = tomSentenceCase
   reCaseToggle = tomToggleCase
End Enum

Enum rePosFlags

   rePosStart = tomStart
   rePosEnd = tomEnd

   rePosTop = 0       ' TA_TOP
   rePosBottom = 8    ' TA_BOTTOM
   rePosBaseLine = 24 ' TA_BASELINE

   rePosLeft = 0      ' TA_LEFT
   rePosRight = 2     ' TA_RIGHT
   rePosCenter = 6    ' TA_CENTER

End Enum

Private Type METAFILEPICT
   mm As Long
   xExt As Long
   yExt As Long
   hMF As Long
End Type

'
' ChangeCase
'
' Changes the text case
'
Public Sub ChangeCase(ByVal New_Case As reChangeCaseFlags)
   m_Range.ChangeCase New_Case
End Sub

'
' Collapse
'
' Collapses the range to the given unit
'
Public Sub Collapse(ByVal Unit As reUnits)
   m_Range.Collapse Unit
End Sub

'
' Copy
'
' Copies the text to the clipboard
'
Public Sub Copy()
   m_Range.Copy 0
End Sub

'
' Cut
'
' Cuts the text and copies it to the clipboard
'
Public Sub Cut()
   m_Range.Cut 0
End Sub

'
' Delete
'
' Deletes text from the range
'
Public Sub Delete(ByVal Unit As reUnits, Optional ByVal Count As Long = 1)
   m_Range.Delete Unit, Count
End Sub

'
' Duplicate
'
' Returns a new Range object that points to the same range.
'
Public Function Duplicate() As Range

   ' Create a new range object
   Set Duplicate = New Range
   
   ' Duplicate this range
   Set Duplicate.Range = m_Range.Duplicate
   
End Function

'
' EndOf
'
' Moves the End position to the end of the last overlapping
' Unit in the range
'
Public Sub EndOf(Optional ByVal Unit As reUnits = reWord, Optional ByVal Extend As Boolean)
   m_Range.EndOf Unit, Abs(Extend)
End Sub

'
' EndPos
'
' Returns the position of the last char in the range
'
Public Property Get EndPos() As Long
   EndPos = m_Range.end
End Property

'
' EndPos
'
' Sets the end position
'
Public Property Let EndPos(ByVal New_EndPos As Long)
   m_Range.end = New_EndPos
End Property

'
' Expand
'
' Expands the range so any partial Unit is
' completely contained
'
Public Function Expand(ByVal Unit As reUnits) As Range
   m_Range.Expand Unit
   Set Expand = Me
End Function

'
' FindEnd
'
' Searches MaxChar characters from the end of the range. If the
' string is found the End position is moved to the end of the
' string.
'
Public Function FindEnd(ByVal Text As String, Optional ByVal flags As reFindFlags, Optional MaxChars As Long = reForward) As Boolean

   FindEnd = m_Range.FindTextEnd(Text, MaxChars, flags) <> 0

End Function

'
' FindEnd
'
' Searches MaxChar characters from the start of the range. If
' the string is found the Start position is moved to the start
' of the string.
'
Public Function FindStart(ByVal Text As String, Optional ByVal flags As reFindFlags, Optional MaxChars As Long = reForward) As Boolean

   FindStart = m_Range.FindTextStart(Text, MaxChars, flags) <> 0

End Function

'
' FindText
'
' Searchs a string in the whole control starting at the range's
' start position. If the string is found the range is collapsed
' to the text.
'
' Returns True if the text was found
'
Public Function FindText(ByVal Text As String, Optional ByVal flags As reFindFlags, Optional MaxChars As Long = reForward) As Boolean

   If m_Range.FindText(Text, MaxChars, flags) <> 0 Then

      FindText = True

      'If Not m_Para Is Nothing Then Set m_Para.frPara = m_Range.Para
      'If Not m_Font Is Nothing Then Set m_Font.frFont = m_Range.Font

   End If

End Function

'
' FindTextRange
'
' Same as FindText but returns a new range with the text found
' or Nothing if the text was not found.
'
Public Function FindTextRange(ByVal Text As String, Optional ByVal flags As reFindFlags, Optional MaxChars As Long = reForward) As Range
Dim Rge As ITextRange

   ' Duplicate the range
   Set Rge = m_Range.Duplicate

   ' Create a new range object
   Set FindTextRange = New Range

   ' Search the string
   If Rge.FindText(Text, MaxChars, flags) <> 0 Then

      ' Create the Range object
      Set FindTextRange = New Range

      ' Set the ITextRange object
      Set FindTextRange.Range = Rge

   End If

End Function

'
' Font
'
Public Property Get Font() As RichFont

   ' Create the Font object if it wasn't created yet
   If m_Font Is Nothing Then
      Set m_Font = New RichFont
      Set m_Font.frFont = m_Range.Font
   End If

   Set Font = m_Font

End Property

'
' Font
'
Public Property Set Font(ByVal New_Font As RichFont)

   ' Set the font of the range
   m_Range.Font.Duplicate = New_Font.frFont

End Property

'
' Range
'
' Sets the ITextRange object
'
Friend Property Set Range(NewRange As ITextRange)

   Set m_Range = NewRange

   'If Not m_Para Is Nothing Then Set m_Para.frPara = m_Range.Para
   'If Not m_Font Is Nothing Then Set m_Font.frFont = m_Range.Font

End Property

'
' Range
'
Friend Property Get Range() As ITextRange
   Set Range = m_Range
End Property

'
' GetPoint
'
' Returns the screen coordinates of the range's
' start or end character
'
Public Sub GetPoint(Optional x As Long, Optional y As Long, Optional ByVal flags As rePosFlags = rePosStart Or rePosBottom Or rePosLeft)
   m_Range.GetPoint flags, x, y
End Sub

'
' InsertImage
'
' Replaces the range with a bitmap or metafile
'
Public Sub InsertImage(ByVal Image As StdPicture)
Dim IDO As IDataObject

   Select Case Image.Type
      Case vbPicTypeBitmap, vbPicTypeMetafile

         ' Store the image
         ' in the global variable
         Set m_Image = Image
   
         ' Get a reference to
         ' the IDataObject_Implements interface
         Set IDO = Me
   
         ' Paste the image
         m_Range.Paste IDO, 0
   
         ' Release the image
         Set m_Image = Nothing

      Case Else
   
         ' Invalid Picture
         Err.Raise 481
        
   End Select

End Sub

'
' InRange
'
' Returns whether a Range is within this range
'
Public Function InRange(ByVal Range As Range) As Boolean
   InRange = m_Range.InRange(Range.Range)
End Function

'
' IsEqual
'
' Returns whether a Range points the same characters
' of this range
'
Public Function IsEqual(ByVal Range As Range) As Boolean
   IsEqual = m_Range.IsEqual(Range.Range)
End Function

'
' Length
'
' Returns the length of the range (in characters)
'
Public Property Get Length() As Long
   Length = m_Range.end - m_Range.Start
End Property

'
' Move
'
' Collapses the range to IP and moves it to the
' next Unit.
'
Public Function Move(Optional ByVal Unit As reUnits = reChar, Optional ByVal Count As Long = 1) As Long
   m_Range.Move Unit, Count
End Function

'
' MoveEnd
'
' Moves the end position
'
Public Function MoveEnd(Optional ByVal Unit As reUnits = reChar, _
                        Optional ByVal Count As Long = 1) As Long
   MoveEnd = m_Range.MoveEnd(Unit, Count)
End Function

'
' MoveEndUntil
'
' Moves the end position until CSet is found
'
Public Function MoveEndUntil(CSet As Variant, _
                        Optional ByVal Count As Long = tomForward) As Long
   MoveEndUntil = m_Range.MoveEndUntil(CSet, Count)
End Function

'
' MoveEndWhile
'
' Moves the end position while CSet is found
'
Public Function MoveEndWhile(CSet As Variant, _
                        Optional ByVal Count As Long = tomForward) As Long
   MoveEndWhile = m_Range.MoveEndWhile(CSet, Count)
End Function

'
' MoveStart
'
Public Function MoveStart(Optional ByVal Unit As reUnits = reChar, Optional ByVal Count As Long = 1) As Long
   MoveStart = m_Range.MoveStart(Unit, Count)
End Function

'
' MoveStartUntil
'
Public Function MoveStartUntil(CSet As Variant, _
                        Optional ByVal Count As Long = tomForward) As Long
   MoveStartUntil = m_Range.MoveStartUntil(CSet, Count)
End Function

'
' MoveStartWhile
'
Public Function MoveStartWhile(CSet As Variant, _
                        Optional ByVal Count As Long = tomForward) As Long
   MoveStartWhile = m_Range.MoveStartWhile(CSet, Count)
End Function

'
' MoveUntil
'
Public Function MoveUntil(CSet As Variant, _
                        Optional ByVal Count As Long = tomForward) As Long
   MoveUntil = m_Range.MoveUntil(CSet, Count)
End Function

'
' MoveWhile
'
Public Function MoveWhile(CSet As Variant, _
                        Optional ByVal Count As Long = tomForward) As Long
   MoveWhile = m_Range.MoveWhile(CSet, Count)
End Function

'
' ParaFormat
'
Public Property Get PARAFORMAT() As ParagraphFormat

   If m_Para Is Nothing Then
      Set m_Para = New ParagraphFormat
      Set m_Para.frPara = m_Range.Para
   End If

   Set PARAFORMAT = m_Para

End Property

'
' ParaFormat
'
Public Property Set PARAFORMAT(ByVal New_ParaFormat As ParagraphFormat)
   
   m_Range.Para.Duplicate = New_ParaFormat.frPara

End Property

'
' Paste
'
' Pastes the context of the clipboard, replacing the
' range.
'
Public Sub Paste(Optional ByVal format As Integer)
   m_Range.Paste 0, format
End Sub

'
' ScrollIntoView
'
' Scrolls the control to show the start or end of
' the range.
'
Public Sub ScrollIntoView(Optional ByVal RangeStart As Boolean = True)

   If RangeStart Then
      m_Range.ScrollIntoView tomStart
   Else
      m_Range.ScrollIntoView tomEnd
   End If

End Sub

'
' SelectRange
'
' Converts the range to the active Selection
'
Public Sub SelectRange()
   m_Range.Select
End Sub

'
' StartOf
'
Public Sub StartOF(Optional ByVal Unit As reUnits = reWord, Optional ByVal Extend As Boolean)
   m_Range.StartOF Unit, Abs(Extend)
End Sub

'
' StartPos
'
' Returns the range's start position
'
Public Property Get StartPos() As Long
   StartPos = m_Range.Start
End Property

'
' StartPos
'
' Sets the range's start position
'
Public Property Let StartPos(ByVal New_StartPos As Long)
   m_Range.Start = New_StartPos
End Property

'
' Text
'
' Returns the text contained by the range
'
Public Property Get Text() As String
   Text = m_Range.Text
End Property

'
' Text
'
' Sets the text contained by the range
'
Public Property Let Text(ByVal New_Text As String)
   m_Range.Text = New_Text
End Property

'
' TextRTF
'
' Returns the range text in RTF format
'
Public Property Get TextRTF() As String
Dim IDO As IDataObject
Dim FMT As FORMATETC
Dim STM As STGMEDIUM
Dim lLen As Long
Dim nFormat As Variant

   nFormat = MyRegisterClipboardFormat(CF_RTF)
   If nFormat > 32768 Then
        nFormat = nFormat - 65536
   End If

   ' Copy the text to a IDataObject_Implements
   m_Range.Copy IDO

   ' Set the FMT structure
   With FMT
      .cfFormat = nFormat 'CF_RTF
      .TYMED = TYMED_HGLOBAL
   End With

   ' Get the text
   IDO.GetData FMT, STM

   ' Get the text length
   lLen = lstrlenA(STM.Data)

   ' Initialize the buffer
   TextRTF = Space$(lLen / 2)

   ' Copy the text to the buffer
   MoveMemory ByVal TextRTF, ByVal STM.Data, lLen

   ' Release the data on STM
   ReleaseStgMedium STM

   ' Convert the string to Unicode
   TextRTF = StrConv(TextRTF, vbUnicode)
   
End Property
'
' TextRTF
'
' Sets the range text in RTF format
'
Public Property Let TextRTF(ByVal New_TextRTF As String)
Dim IDO As IDataObject
Dim nFormat As Variant


   ' Convert the string to ANSI
   m_IDORTF = StrConv(New_TextRTF, vbFromUnicode)

   Set IDO = Me

MsgBox "Range.Let TextRTF 1"
   
   nFormat = MyRegisterClipboardFormat(CF_RTF)
   If nFormat > 32768 Then
        nFormat = nFormat - 65536
   End If
MsgBox "Range.Let TextRTF 2"
   
   m_Range.Paste IDO, 1 ' CF_RTF
MsgBox "Range.Let TextRTF 3"

   m_IDORTF = vbNullString

End Property

Private Sub IDataObject_DAdvise(pformatetc As olelib.FORMATETC, ByVal ADVF As Long, ByVal pAdvSink As Long, pdwConnection As Long)
   Err.Raise E_NOTIMPL
End Sub

Private Sub IDataObject_DUnadvise(ByVal dwConnection As Long)
   Err.Raise E_NOTIMPL
End Sub

Private Sub IDataObject_EnumDAdvise(pIEnumAdvise As Long)
   Err.Raise E_NOTIMPL
End Sub

Private Function IDataObject_EnumFormatEtc(ByVal dwDirection As olelib.DATADIR) As olelib.IEnumFORMATETC
   Err.Raise E_NOTIMPL
End Function

Private Sub IDataObject_GetCanonicalFormatEtc(pformatectIn As FORMATETC, pformatetcOut As FORMATETC)
   Err.Raise E_NOTIMPL
End Sub

Private Sub IDataObject_GetData(pformatetcIn As FORMATETC, pmedium As STGMEDIUM)
   Err.Raise E_NOTIMPL
End Sub

Private Sub IDataObject_GetDataHere(pformatetc As FORMATETC, pmedium As STGMEDIUM)
   Err.Raise E_NOTIMPL
End Sub

Private Sub IDataObject_QueryGetData(pformatetc As FORMATETC)
   Err.Raise E_NOTIMPL
End Sub

Private Sub IDataObject_SetData(pformatetc As olelib.FORMATETC, pmedium As olelib.STGMEDIUM, ByVal fRelease As Long)
   Err.Raise E_NOTIMPL
End Sub

Private Sub Class_Terminate()

   ' Destroy all objects
   Set m_Range = Nothing
   Set m_Font = Nothing
   Set m_Para = Nothing

End Sub

