VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Selection"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Edanmo's RichEdit Control v3 - Selection Class"
'*********************************************************************************************
'
' RichEdit Control
'
' Selection class
'
'*********************************************************************************************
'
' Author: Eduardo A. Morcillo
' E-Mail: e_morcillo@yahoo.com
' Web Page: http://www.domaindlx.com/e_morcillo
'
' Distribution: You can freely use this code in your own applications but you
'               can't publish this code in a web site, online service, or any
'               other media, without my express permission.
'
' Usage: at your own risk.
'
' Tested on: Windows 98 + VB5
'
' History:
'           07/11/2000 - This code was released
'
'*********************************************************************************************
Option Explicit

' Selection types
Enum reSelectionTypes
   reNoSelection        ' There's no selection and no insertion point
   reSelectionIP        ' The selection is the insertion point
   reSelectionNormal    ' Text is selected
   reSelectionFrame     ' Frame
   reSelectionColumn    ' Table column
   reSelectionRow       ' Table row
   reSelectionBlock     ' Block selection
   reSelectionObject    ' Embedded object/Picture
   reSelectionShape     ' Shape
End Enum

Dim m_Sel As ITextSelection
Dim m_hWnd As Long
Dim m_Range As Range

Public Sub EndKey(Optional ByVal Unit As reUnits = reLine, Optional ByVal ExtendSelection As Boolean)
   m_Sel.EndKey Unit, ExtendSelection
End Sub

'
' Link
'
' Returns whether the selection is a link
'
Public Property Get Link() As Boolean
Dim CF2 As CHARFORMAT2

   With CF2
      .cbSize = LenB(CF2)
      .dwMask = CFM_LINK
   End With

   SendMessage m_hWnd, EM_GETCHARFORMAT, 1, CF2

   Link = (CF2.dwEffects And CFE_LINK) = CFE_LINK

End Property

'
' Link
'
' Sets whether the selection is a link
'
Public Property Let Link(ByVal New_Link As Boolean)
Dim CF2 As CHARFORMAT2

   With CF2
      .cbSize = LenB(CF2)
      .dwMask = CFM_LINK
      .dwEffects = -CFE_LINK * New_Link
   End With

   SendMessage m_hWnd, EM_SETCHARFORMAT, SCF_SELECTION, CF2

End Property

'
' OutlineExpand
'
' Expands/Collapses the selection in Outline mode
'
' Requires RE v3
'
Public Sub OutlineExpand(Optional ByVal Levels As Integer = 1)

   SendMessage m_hWnd, EM_OUTLINE, EMO_EXPAND, ByVal CLng(Levels)

End Sub

'
' frSetData
'
' Sets the internal data
'
Friend Sub frInitialize(ByVal hWnd As Long, ByVal Rng As ITextSelection)

   ' Set the ITextSelection object
   Set m_Sel = Rng

   ' Save the richedit window
   m_hWnd = hWnd

End Sub

'
' HomeKey
'
Public Sub HomeKey(Optional ByVal Unit As reUnits = reLine, Optional ByVal ExtendSelection As Boolean)
   m_Sel.HomeKey Unit, ExtendSelection
End Sub

'
' MoveDown
'
' Moves the insertion point down
'
Public Sub MoveDown(Optional ByVal Unit As reUnits = reLine, Optional ByVal Count As Long = 1, Optional ByVal ExtendSelection As Boolean)
   m_Sel.MoveDown Unit, Count, ExtendSelection
End Sub

'
' MoveLeft
'
' Moves the insertion point to the left
'
Public Sub MoveLeft(Optional ByVal Unit As reUnits = reLine, Optional ByVal Count As Long = 1, Optional ByVal ExtendSelection As Boolean)
   m_Sel.MoveLeft Unit, Count, ExtendSelection
End Sub

'
' MoveRight
'
' Moves the insertion point to the right
'
Public Sub MoveRight(Optional ByVal Unit As reUnits = reLine, Optional ByVal Count As Long = 1, Optional ByVal ExtendSelection As Boolean)
   m_Sel.MoveRight Unit, Count, ExtendSelection
End Sub

'
' MoveUp
'
' Moves the insertion point up
'
Public Sub MoveUp(Optional ByVal Unit As reUnits = reLine, Optional ByVal Count As Long = 1, Optional ByVal ExtendSelection As Boolean)
   m_Sel.MoveUp Unit, Count, ExtendSelection
End Sub

'
' OutlinePromote
'
' Promotes/Demotes the selection
'
' Requires RE v3
'
Public Sub OutlinePromote(Optional ByVal Levels As Integer = 1)

   SendMessage m_hWnd, EM_OUTLINE, EMO_PROMOTE, ByVal CLng(Levels)

End Sub

'
' Range
'
' Returns the selection range
'
Public Property Get Range() As Range

   If m_Range Is Nothing Then
      Set m_Range = New Range
      Set m_Range.Range = m_Sel
   End If

   Set Range = m_Range

End Property

Public Property Get RightToLeft() As Boolean

'Dim PF2 As ParafORMAT2

'   With CF2
'      .cbSize = LenB(CF2)
'      .dwMask = CFM_LINK
'      .dwEffects = -CFE_LINK * New_Link
'   End With

'   SendMessage m_hWnd, EM_SETCHARFORMAT, SCF_SELECTION, CF2

End Property

'
' SelType
'
' Returns the selection type
'
Public Property Get SelType() As reSelectionTypes
Attribute SelType.VB_UserMemId = 0
   SelType = m_Sel.Type
End Property

'
' TypeText
'
' Types the string at this selection as if
' someone typed it.
'
Public Sub TypeText(ByVal Text As String)
   m_Sel.TypeText Text
End Sub
Public Sub ReplaceText(ByVal Text As String)
   m_Sel.Text = Text
End Sub


Private Sub Class_Terminate()

   ' Destroy all objects
   Set m_Range = Nothing
   Set m_Sel = Nothing

End Sub
